disp(' ')
disp(' sdof_ran.m   ver 1.3   June 18, 2007')
disp(' by Tom Irvine   Email: tomirvine@aol.com')
disp(' ')
disp(' This program calculates the response of a single-degree-of-freeom system')
disp(' to an acceleration power spectral density')
disp(' ')
disp(' The power spectral density may vary arbitrarily with frequency. ')
disp(' The input file must have two columns: frequency(Hz) & accel(G^2/Hz)')
disp(' ')
%
clear f;
clear a;
clear s;
clear THF;
%
disp(' Select file input method       ');
disp('   1=external ASCII file        ');
disp('   2=file preloaded into Matlab ');
disp('   3=library function           ');
file_choice = input('');
%
if(file_choice==1)
    [filename, pathname] = uigetfile('*.*');
    filename = fullfile(pathname, filename);
    fid = fopen(filename,'r');
    THF = fscanf(fid,'%g %g',[2 inf]);
    THF=THF';
end
if(file_choice==2)
    THF = input(' Enter the matrix name:  ');
end
if(file_choice==1 | file_choice==2)
    size(THF);
    f=THF(:,1);
    a=THF(:,2);
end
%
if(file_choice==3)
    disp(' Select input PSD      ');
    disp('   1=MIL-STD-1540B QTP ');
    disp('   2=MIL-STD-1540B ATP ');
    disp('   3=NAVMAT P9495      ');
    disp('   4=NASA CEQATR AVT   '); 
    ilibrary=input(' ');
%
	if(ilibrary==1 | ilibrary==2 )
	    f(1)=20.;	
		a(1)=0.0053;
		f(2)=150.;	
		a(2)=0.04;
		f(3)=600.;
		a(3)=0.04;
		f(4)=2000.; 
		a(4)=0.0036;
		n=4;
		Lflag=1;
    end
	if(ilibrary==1)
		a(1)=a(1)*4.;
		a(2)=a(2)*4.;
		a(3)=a(3)*4.;
		a(4)=a(4)*4.;
    end
	if(ilibrary==3)
		f(1)=20.;	
		a(1)=0.01;
		f(2)=80.;	
		a(2)=0.04;
		f(3)=350.;
		a(3)=0.04;
		f(4)=2000.; 
		a(4)=0.007;
    end 
   	if(ilibrary==4)
		f(1)=20.;	
		a(1)=0.01;
		f(2)=80.;	
		a(2)=0.04;
		f(3)=800.;
		a(3)=0.04;
		f(4)=2000.; 
		a(4)=0.01;
    end    
%
end
%
MAX = 5000;
tpi=2.*pi;
%
disp(' ');
natural_frequency=input(' Enter the natural frequency (Hz)  ');
%
idamp=input(' Enter damping format:  1= damping ratio   2= Q  ');	
%
disp(' ')
if(idamp==1)
    damp=input(' Enter damping ratio (typically 0.05) ');
else
    Q=input(' Enter the amplification factor (typically Q=10) ');
    damp=1./(2.*Q);
end
%
n=length(f);
%
[s,grms]=calculate_PSD_slopes(f,a);
out5 = sprintf('\n Input PSD Overall Level = %10.3f ',grms);
disp(out5);
%
df=1.;
%
[fi,ai]=interpolate_PSD(f,a,s,df);
%
[a_vrs,rd_vrs,trans,opsd]=sdof_ran_engine(fi,ai,damp,df,natural_frequency);
%
disp(' ')
Q=1./(2.*damp);
%
disp(' ')
disp(' SDOF Acceleration Response ')
disp(' ')
out1=sprintf('     = %8.3g GRMS',a_vrs);
out2=sprintf('     = %8.3g G 3-sigma',3.*a_vrs);
disp(out1);
disp(out2);
disp(' ')
disp(' SDOF Relative Displacement Response ')
disp(' ')
out1=sprintf('     = %8.3g inch RMS',rd_vrs);
out2=sprintf('     = %8.3g inch 3-sigma',3.*rd_vrs);
disp(out1);
disp(out2);
%
disp(' ');
disp(' Plot the output PSD? ')
choice = input(' 1=yes 2=no ');
%
if(choice == 1)
%
        plot(fi,opsd,f,a,'-.'); 
        figure(1);  
        legend ('Response','Input',2);       
        xlabel(' Frequency (Hz)');
        ylabel(' Accel (G^2/Hz)'); 
        at = sprintf(' Power Spectral Density   fn=%7.3g Hz  Q=%5g  Level=%6.3g GRMS',natural_frequency,Q,a_vrs);   
        title(at);
        set(gca,'MinorGridLineStyle','none','GridLineStyle',':','XScale','log','YScale','log'); 
        grid;       
%        
end
%
disp(' ');
disp(' Plot the power transmissibility function? ')
choice = input(' 1=yes 2=no ');
%
if(choice == 1)
%        
        clear figure(1);
        plot(fi,trans);
        figure(1);   
        xlabel(' Frequency (Hz)');
        ylabel(' Trans (G^2/G^2)'); 
        at = sprintf(' Power Transmissibility   fn=%7.3g Hz  Q=%5g',natural_frequency,Q);   
        title(at);
        set(gca,'MinorGridLineStyle','none','GridLineStyle',':','XScale','log','YScale','log'); 
        grid;       
%        
end   
